@@ -45,9 +45,8 @@ cbind.mids <- function(x, y = NULL, ...) {
   blocknames <- make.unique(xynames)
   names(blocknames) <- xynames
   names(blocks) <- blocknames
-  ct <- c(attr(x$blocks, "calltype"), rep("type", ncol(y)))
-  names(ct) <- blocknames
-  attr(blocks, "calltype") <- ct
+  calltype <- c(x$calltype, rep("pred", ncol(y)))
+  names(calltype) <- blocknames
 
   m <- x$m
 
@@ -60,9 +59,9 @@ cbind.mids <- function(x, y = NULL, ...) {
   r <- (!is.na(y))
   f <- function(j) {
     m <- matrix(NA,
-      nrow = sum(!r[, j]),
-      ncol = x$m,
-      dimnames = list(row.names(y)[!r[, j]], seq_len(m))
+                nrow = sum(!r[, j]),
+                ncol = x$m,
+                dimnames = list(row.names(y)[!r[, j]], seq_len(m))
     )
     as.data.frame(m)
   }
@@ -79,15 +78,15 @@ cbind.mids <- function(x, y = NULL, ...) {
   predictorMatrix <- rbind(
     x$predictorMatrix,
     matrix(0,
-      ncol = ncol(x$predictorMatrix),
-      nrow = ncol(y)
+           ncol = ncol(x$predictorMatrix),
+           nrow = ncol(y)
     )
   )
   predictorMatrix <- cbind(
     predictorMatrix,
     matrix(0,
-      ncol = ncol(y),
-      nrow = nrow(x$predictorMatrix) + ncol(y)
+           ncol = ncol(y),
+           nrow = nrow(x$predictorMatrix) + ncol(y)
     )
   )
   rownames(predictorMatrix) <- blocknames
@@ -111,14 +110,19 @@ cbind.mids <- function(x, y = NULL, ...) {
   loggedEvents <- x$loggedEvents
 
   ## save, and return
-  midsobj <- list(
-    data = data, imp = imp, m = m,
-    where = where, blocks = blocks,
-    call = call, nmis = nmis,
+  midsobj <- mids(
+    data = data,
+    imp = imp,
+    m = m,
+    where = where,
+    blocks = blocks,
+    call = call,
+    nmis = nmis,
     method = method,
     predictorMatrix = predictorMatrix,
     visitSequence = visitSequence,
     formulas = formulas,
+    calltype = calltype,
     post = post,
     blots = blots,
     ignore = ignore,
@@ -127,12 +131,8 @@ cbind.mids <- function(x, y = NULL, ...) {
     lastSeedValue = lastSeedValue,
     chainMean = chainMean,
     chainVar = chainVar,
-    loggedEvents = loggedEvents,
-    version = packageVersion("mice"),
-    date = Sys.Date()
-  )
-  oldClass(midsobj) <- "mids"
-  midsobj
+    loggedEvents = loggedEvents)
+  return(midsobj)
 }
 
 
@@ -173,9 +173,8 @@ cbind.mids.mids <- function(x, y, call) {
   blocknames <- make.unique(xynames)
   names(blocknames) <- xynames
   names(blocks) <- blocknames
-  ct <- c(attr(xblocks, "calltype"), attr(yblocks, "calltype"))
-  names(ct) <- blocknames
-  attr(blocks, "calltype") <- ct
+  calltype <- c(x$calltype, y$calltype)
+  names(calltype) <- blocknames
 
   m <- x$m
   nmis <- c(x$nmis, y$nmis)
@@ -185,21 +184,37 @@ cbind.mids.mids <- function(x, y, call) {
   method <- c(x$method, y$method)
   names(method) <- blocknames
 
+  # Concatenate formulas, rename variables if needed
+  if (all(names(ynew) == unname(ynew)) && all(names(xnew) == unname(xnew))) {
+    formulas <- c(x$formulas, y$formulas)
+  } else {
+    xformulas <- x$formulas
+    yformulas <- y$formulas
+    for (i in names(xformulas)) {
+      xformulas[[i]] <- renf(xformulas[[i]], xnew)
+    }
+    for (i in names(yformulas)) {
+      yformulas[[i]] <- renf(yformulas[[i]], ynew)
+    }
+    formulas <- c(xformulas, yformulas)
+  }
+  names(formulas) <- blocknames
+
   # The predictorMatrices of x and y are combined with zero matrices
   # on the off diagonal blocks.
   predictorMatrix <- rbind(
     x$predictorMatrix,
     matrix(0,
-      ncol = ncol(x$predictorMatrix),
-      nrow = nrow(y$predictorMatrix)
+           ncol = ncol(x$predictorMatrix),
+           nrow = nrow(y$predictorMatrix)
     )
   )
   predictorMatrix <- cbind(
     predictorMatrix,
     rbind(
       matrix(0,
-        ncol = ncol(y$predictorMatrix),
-        nrow = nrow(x$predictorMatrix)
+             ncol = ncol(y$predictorMatrix),
+             nrow = nrow(x$predictorMatrix)
       ),
       y$predictorMatrix
     )
@@ -212,9 +227,6 @@ cbind.mids.mids <- function(x, y, call) {
   xnew <- blocknames[1:length(x$blocks)]
   ynew <- blocknames[-(1:length(x$blocks))]
   visitSequence <- unname(c(xnew[x$visitSequence], ynew[y$visitSequence]))
-
-  formulas <- c(x$formulas, y$formulas)
-  names(formulas) <- blocknames
   post <- c(x$post, y$post)
   names(post) <- varnames
   blots <- c(x$blots, y$blots)
@@ -269,14 +281,19 @@ cbind.mids.mids <- function(x, y, call) {
 
   loggedEvents <- x$loggedEvents
 
-  midsobj <- list(
-    data = data, imp = imp, m = m,
-    where = where, blocks = blocks,
-    call = call, nmis = nmis,
+  midsobj <- mids(
+    data = data,
+    imp = imp,
+    m = m,
+    where = where,
+    blocks = blocks,
+    call = call,
+    nmis = nmis,
     method = method,
     predictorMatrix = predictorMatrix,
     visitSequence = visitSequence,
     formulas = formulas,
+    calltype = calltype,
     post = post,
     blots = blots,
     ignore = ignore,
@@ -285,10 +302,16 @@ cbind.mids.mids <- function(x, y, call) {
     lastSeedValue = lastSeedValue,
     chainMean = chainMean,
     chainVar = chainVar,
-    loggedEvents = loggedEvents,
-    version = packageVersion("mice"),
-    date = Sys.Date()
-  )
-  oldClass(midsobj) <- "mids"
-  midsobj
+    loggedEvents = loggedEvents)
+  return(midsobj)
+}
+
+renf <- function(f, nn) {
+  # rename variables in formula f
+  z <- as.character(f)
+  for (i in seq_along(nn)) {
+    z <- gsub(names(nn)[i], unname(nn)[i], z)
+  }
+  nf <- formula(paste(z[2], z[1], z[3], collapse = " "))
+  return(nf)
 }
